@@ -29,6 +29,7 @@ from keystonemiddleware import fixture as ksm_fixture
 import testscenarios
 from testtools import testcase
 from unittest import mock
+import webob
 import webtest
 
 import gnocchi
@@ -333,20 +334,75 @@ class MetricTest(RestTest):
             self.assertNotIn(metric_id, [m["id"] for m in metric_list.json])
 
     def test_list_metric_with_another_user_allowed(self):
-        rid = str(uuid.uuid4())
-        r = self.app.post_json("/v1/resource/generic",
-                               params={
-                                   "id": rid,
-                                   "project_id": TestingApp.PROJECT_ID_2,
-                                   "metrics": {
-                                       "disk": {"archive_policy_name": "low"},
-                                   }
-                               })
-        metric_id = r.json['metrics']['disk']
+        r1id = str(uuid.uuid4())
+        r1 = self.app.post_json("/v1/resource/generic",
+                                params={
+                                    "id": r1id,
+                                    "project_id": TestingApp.PROJECT_ID,
+                                    "metrics": {
+                                        "disk": {
+                                            "archive_policy_name": "low"},
+                                    }
+                                })
+        metric_id1 = r1.json['metrics']['disk']
+        r2id = str(uuid.uuid4())
+        r2 = self.app.post_json("/v1/resource/generic",
+                                params={
+                                    "id": r2id,
+                                    "project_id": TestingApp.PROJECT_ID_2,
+                                    "metrics": {
+                                        "disk": {
+                                            "archive_policy_name": "low"},
+                                    }
+                                })
+        metric_id2 = r2.json['metrics']['disk']
 
         with self.app.use_another_user():
-            metric_list = self.app.get("/v1/metric")
-            self.assertIn(metric_id, [m["id"] for m in metric_list.json])
+            metric_list = self.app.get("/v1/metric").json
+            metric_ids = [m["id"] for m in metric_list]
+            self.assertNotIn(metric_id1, metric_ids)
+            self.assertIn(metric_id2, metric_ids)
+
+    def test_list_metric_with_another_user_allowed_no_creator_policy(self):
+        r1id = str(uuid.uuid4())
+        r1 = self.app.post_json("/v1/resource/generic",
+                                params={
+                                    "id": r1id,
+                                    "project_id": TestingApp.PROJECT_ID,
+                                    "metrics": {
+                                        "disk": {
+                                            "archive_policy_name": "low"},
+                                    }
+                                })
+        metric_id1 = r1.json['metrics']['disk']
+        r2id = str(uuid.uuid4())
+        r2 = self.app.post_json("/v1/resource/generic",
+                                params={
+                                    "id": r2id,
+                                    "project_id": TestingApp.PROJECT_ID_2,
+                                    "metrics": {
+                                        "disk": {
+                                            "archive_policy_name": "low"},
+                                    }
+                                })
+        metric_id2 = r2.json['metrics']['disk']
+
+        orig_enforce = api.enforce
+
+        def _enforce(rule, target):
+            if rule == "list metric" and "created_by_project_id" in target:
+                raise webob.exc.HTTPForbidden()
+            return orig_enforce(rule, target)
+
+        with mock.patch.object(api,
+                               'enforce',
+                               side_effect=_enforce) as mock_enforce:
+            with self.app.use_another_user():
+                metric_list = self.app.get("/v1/metric").json
+                metric_ids = [m["id"] for m in metric_list]
+                self.assertNotIn(metric_id1, metric_ids)
+                self.assertIn(metric_id2, metric_ids)
+                mock_enforce.assert_called()
 
     def test_get_metric_with_another_user(self):
         result = self.app.post_json("/v1/metric",
@@ -1500,7 +1556,7 @@ class ResourceTest(RestTest):
         result = self.app.get("/v1/resource/generic?details=awesome",
                               status=400)
         self.assertIn(
-            b"Unable to parse `details': invalid truth value",
+            b"Unable to parse `details':",
             result.body)
 
     def _do_test_list_resources_with_detail(self, request):
@@ -1636,7 +1692,7 @@ class ResourceTest(RestTest):
             params={"=": {"name": name}},
             status=400,
             headers={"Accept": "application/json"})
-        self.assertEqual("Metrics can't being aggregated",
+        self.assertEqual("Metrics cannot be aggregated",
                          result.json['description']['cause'])
         self.assertEqual("No granularity match",
                          result.json['description']['reason'])
@@ -1886,6 +1942,58 @@ class GenericResourceTest(RestTest):
         )
 
 
+class AggregatesTest(RestTest):
+    def test_get_metric_aggregates_with_another_user(self):
+        r = self.app.post_json(
+            "/v1/metric",
+            params={"archive_policy_name": "medium"},
+            status=201)
+        metric_id = r.json['id']
+        self.app.post_json(
+            f"/v1/metric/{metric_id}/measures",
+            params=[{"timestamp": "2013-01-01 12:00:01",
+                     "value": 8},
+                    {"timestamp": "2013-01-01 12:00:02",
+                     "value": 16}])
+        with self.app.use_another_user():
+            self.app.post_json(
+                "/v1/aggregates",
+                params={"operations": ["metric", metric_id, "mean"]},
+                status=403)
+
+    def test_get_metric_aggregates_with_another_user_allowed(self):
+        rid = str(uuid.uuid4())
+        r = self.app.post_json(
+            "/v1/resource/generic",
+            params={
+                "id": rid,
+                "project_id": TestingApp.PROJECT_ID_2,
+                "metrics": {
+                    "disk": {"archive_policy_name": "low"},
+                }
+            })
+        metric_id = r.json['metrics']['disk']
+        self.app.post_json(
+            f"/v1/metric/{metric_id}/measures",
+            params=[{"timestamp": "2013-01-01 12:00:01",
+                     "value": 8},
+                    {"timestamp": "2013-01-01 12:00:02",
+                     "value": 16}])
+        with self.app.use_another_user():
+            r = self.app.post_json(
+                "/v1/aggregates",
+                params={"operations": ["metric", metric_id, "mean"]},
+                status=200)
+            aggregates = r.json
+            self.assertIn(metric_id, aggregates["measures"])
+            measures = aggregates["measures"][metric_id]
+            self.assertIn("mean", measures)
+            self.assertEqual([["2013-01-01T00:00:00+00:00", 86400, 12],
+                              ["2013-01-01T12:00:00+00:00", 3600, 12],
+                              ["2013-01-01T12:00:00+00:00", 300, 12]],
+                             measures["mean"])
+
+
 class QueryStringSearchAttrFilterTest(tests_base.TestCase):
     def _do_test(self, expr, expected):
         req = api.QueryStringSearchAttrFilter._parse(expr)
